#include <qpainter.h>
#include <qpixmap.h>
#include <qpushbt.h>
#include <qlayout.h>
#include <qmsgbox.h>
#include <qfiledlg.h>
#include <qtooltip.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <err.h>
#include "ListView.h"
#include "filebrowse.h"
#include "unknown.res"

#define LINE_BUF 101

/*
void AlbumListBoxItem::paint( QPainter *p )
{
	p->drawPixmap( 3, 0, pm );
	QFontMetrics fm = p->fontMetrics();
	int yPos;                       		// vertical text position
	if ( pm.height() < fm.height() )
            	yPos = fm.ascent() + fm.leading()/2;
        else
  		yPos = pm.height()/2 - fm.height()/2 + fm.ascent();
        p->drawText( pm.width() + 5, yPos, text() );
}

int AlbumListBoxItem::height(const QListBox *lb ) const
{
        return QMAX( pm.height(), lb->fontMetrics().lineSpacing() + 1 );
}

int AlbumListBoxItem::width(const QListBox *lb ) const
{
        return pm.width() + lb->fontMetrics().width( text() ) + 6;
}
*/


void AlbumItem::AddSong(char *song, char *locate)
{
	SongList.append(song);
	FileList.append(locate);
	total++;
}

void AlbumItem::AddSong(QString song, QString locate)
{
	SongList.append(song.data());
	FileList.append(locate.data());
	total++;
}


AlbumItem& AlbumItem::operator=(const AlbumItem *i)
{
        Singer = i->Singer;
        Publish = i->Publish;
        Picture = i->Picture;
        SingerAlbum = i->SingerAlbum;
        SongList = i->SongList;
        FileList = i->FileList;
        total = i->total;
        return *this;
}

AlbumListBox::AlbumListBox(QWidget* parent=0, const char* name=0)
		:MultiListBox(parent,(char *)name)
{
	ptr.setAutoDelete(TRUE);
	total=0;
	AlbumChange = 0;
	sprintf(filename,"%s/.album.dat",getenv("HOME"));
	fun = new QPopupMenu();
	fun->insertItem("&New",this,SLOT(NewAlbum()));
	fun->insertItem("&Edit",this,SLOT(EditAlbum()));
	fun->insertItem("&Delete",this,SLOT(DelAlbum()));
	fun->insertItem("&Save",this,SLOT(SaveToFile()));
}

void AlbumListBox::NewAlbum()
{
	AlbumItem *it;
	it = NULL;
	AlbumInfoView *info = new AlbumInfoView(it,-1,this);	//-1: nosense ?
	if (info->exec() == QDialog::Accepted){
		AlbumItem a = info->GetAlbumInfo();
		DoAppendAlbum(&a);
	}
	delete info;
}

void AlbumListBox::EditAlbum()
{
	AlbumInfoView *info = new AlbumInfoView(ptr.at(CurrentItem()),CurrentItem(),this);
	if (info->exec() == QDialog::Accepted){
		AlbumItem a = info->GetAlbumInfo();
		DoInsertAlbum(info->GetAlbumIndex(),&a);
	}
	delete info;
}

void AlbumListBox::DelAlbum()
{
	if (CurrentItem() > -1){
		ptr.remove(CurrentItem());
		total--;
		DeleteItem(CurrentItem());
		AlbumChange = 1;
	}
}


void AlbumListBox::mousePressEvent(QMouseEvent *e)
{
//	printf("press (%d)\n",e->button());
	MultiListBox::mousePressEvent(e);
	if (e->button() == RightButton)
		fun->popup(mapToGlobal(e->pos()));
}

void AlbumListBox::mouseReleaseEvent(QMouseEvent *e)
{
//	printf("release (%d)\n",e->button());
	MultiListBox::mouseReleaseEvent(e);
	if (e->button() == RightButton)
		fun->hide();
}

void AlbumListBox::mouseDoubleClickEvent( QMouseEvent *e )
{
	mouseReleaseEvent(e);
	if ( CurrentItem() > -1 ){
		AlbumItem info;
		info = ptr.at(CurrentItem());
		emit selected(info);
	}
}

int AlbumListBox::LoadFromFile()
{
	FILE *fp;
	char buf[101];
	char *header,*value,*songname,*location;
	int begin;
	AlbumItem *al;

	if (access(filename,R_OK) == -1)
		return 0;

	if ((fp=fopen(filename,"r")) == NULL){
		QMessageBox::warning(this,"error","can't open configuration file\n");
		return 0;
	}
	Clear();
	begin = 0 ;
	while(fgets(buf,LINE_BUF,fp) != NULL){
		if (buf[0] == ' ' || buf[0] == '\n')
			continue;		// pass blank line between [item]

		header = strtok(buf,"=\n");
		value = strtok(NULL,"\n");
//		printf("%s=%s\n",header,value);

		if (strstr(buf,"[Item]") != NULL){
			if (begin == 1) {
				ptr.append(al);
				total++;
				begin = 0;
			}
			al = new AlbumItem();
			begin = 1;
		}
		else if (strstr(header,"singer") != NULL)
			al->Singer=value;
		else if (strstr(header,"album") != NULL)
			al->SingerAlbum=value;
		else if (strstr(header,"publish") != NULL)
			al->Publish=value;
		else if (strstr(header,"picture") != NULL)
			al->Picture=value;
		else if (strstr(header,"song") != NULL){
			songname = strtok(value,"|\n");
			location = strtok(NULL,"\n");
			if (location == NULL)
				location=songname;	// if no song name, location instead 
			al->AddSong(songname,location);
		}
		else
			printf("Unknown Data:(%s=%s)\n",header,value);
	}
	if (begin == 1){
		ptr.append(al);
		total++;
		begin = 0;
	}
	fclose(fp);
	Refresh();
	return 1;
}

int AlbumListBox::SaveToFile()
{
	AlbumItem *p;
	QStrList *song;
	QStrList *file;
	char *s,*f;
	int i,j,no;
	FILE *fp;
	QString SaveStream;

	if ((fp=fopen(filename,"w+")) == NULL){
		QMessageBox::warning(this,"error","can't open configuration file\n");
		return 0;
	}

	for (p=ptr.first();p != NULL;p=ptr.next()){
		song = (QStrList *)p->GetSongList();
		file = (QStrList *)p->GetFileList();
		no = file->count();		// base on file list

		SaveStream.append("\n[Item]\nsinger=");		// Header
		SaveStream.append(p->GetSinger());
		SaveStream.append("\nalbum=");
		SaveStream.append(p->GetSingerAlbum());
		SaveStream.append("\npublish=");
		SaveStream.append(p->GetPublish());
		SaveStream.append("\npicture=");
		SaveStream.append(p->GetPicture());
		for (s=song->first(),f=file->first();s != NULL;s=song->next(),f=file->next()){
			SaveStream.append("\nsong=");
			SaveStream.append(s);
			SaveStream.append("|");
			SaveStream.append(f);
		}
	}
	fprintf(fp,"%s\n",SaveStream.data());
        AlbumChange = 0;
	fclose(fp);
	return 1;
}

void AlbumListBox::DumpInfo()
{
	AlbumItem *p;
	QStrList *song;
	QStrList *file;
	char *s,*f;
	int i,j,no;

	for (p=ptr.first();p != NULL;p=ptr.next()){
		song = (QStrList *)p->GetSongList();
		file = (QStrList *)p->GetFileList();
		no = file->count();		// base on file list

		printf("\n[Item]\n");		// Header
		printf("singer=%s\n",p->GetSinger());
		printf("album=%s\n",p->GetSingerAlbum());
		printf("publish=%s\n",p->GetPublish());
		printf("picture=%s\n",p->GetPicture());
		for (s=song->first(),f=file->first();s != NULL;s=song->next(),f=file->next())
			printf("song=%s|%s\n",s,f);
		printf("Total: %d songs\n",no);
	}
	printf("Total Album: %d\n",total);
}

void AlbumListBox::DoAppendAlbum(AlbumItem *data)
{
	AlbumItem *al = new AlbumItem();
	*al = data;
	ptr.append(al);
	total++;

	char buf[51];
	MultiListBoxItem *item;
	QPixmap pixmap;
	sprintf(buf,"%s (%s)",!al->SingerAlbum.isEmpty()?al->GetSingerAlbum():"Unknown",!al->Singer.isEmpty()?al->GetSinger():"Unknown");
	if ((pixmap.load(al->GetPicture())) == FALSE)
		pixmap = QPixmap((const char **)unknown_album_xpm);
	item = new MultiListBoxItem(pixmap,buf);

	AppendItem(item);
	AlbumChange = 1;
}

void AlbumListBox::DoInsertAlbum(int i,AlbumItem *data)
{
	AlbumItem *al = new AlbumItem();
	*al = data;
	ptr.remove(i);
	ptr.insert(i,al);

	char buf[51];
	MultiListBoxItem *item;
	QPixmap pixmap;
	sprintf(buf,"%s (%s)",!al->SingerAlbum.isEmpty()?al->GetSingerAlbum():"Unknown",!al->Singer.isEmpty()?al->GetSinger():"Unknown");
	if ((pixmap.load(al->GetPicture())) == FALSE)
		pixmap = QPixmap((const char **)unknown_album_xpm);
	item = new MultiListBoxItem(pixmap,buf);	

	ChangeItem(i,item);	
	AlbumChange = 1;
}

void AlbumListBox::Refresh()
{
	int i;
	AlbumItem *al;
	char buf[51];

	Clear();
//	printf("refresh\n");
	for (i=0,al=ptr.first();i<total;i++,al=ptr.next()){
		MultiListBoxItem *item;
		QPixmap pixmap;
		sprintf(buf,"%s (%s)",!al->SingerAlbum.isEmpty()?al->GetSingerAlbum():"Unknown",!al->Singer.isEmpty()?al->GetSinger():"Unknown");
		if ((pixmap.load(al->GetPicture())) == FALSE)
			pixmap = QPixmap((const char **)unknown_album_xpm);
		item = new MultiListBoxItem(pixmap,buf);
		AppendItem(item);
	}
//	DumpInfo();
}

Mixer::Mixer()
{
	const char *mn[SOUND_MIXER_NRDEVICES] = SOUND_DEVICE_NAMES;
	names = (const char **)mn;
	mixer = MIXER;
	if ((fd = open(mixer, O_RDWR)) < 0)
		QMessageBox::warning(0,"error","can't open /dev/mixer\n");
}

int Mixer::Get(void)
{
	int state,r,l;

	if (ioctl(fd, MIXER_READ(DEVICE),&state)== -1) {
		warn("MIXER_READ");
		return (-1);
	}
	else {
		r = state & 0x7f ;
		l = (state >> 8) & 0x7f ;
		return (r > l) ? r : l ;
	}
}
	
int Mixer::Set(int l)
{
	int r;

	if (l<0)
		l=0;
	if (l>100)
		l=100;
	r = l;
	l |= r << 8;
	if (ioctl(fd, MIXER_WRITE(DEVICE), &l) == -1)
		return (-1);
	return(1);
}

AlbumInfoView::AlbumInfoView(AlbumItem *item,int i,QWidget *parent=0)
		:QDialog(parent,"Info",TRUE)
{
	QLabel *lbl;
	QPushButton *btn;

	if ( i > -1 )
		setCaption("Edit Album");
	else
		setCaption("New Album");
	QGridLayout *topLayout = new QGridLayout(this,16,4,5);

        AlbumItem temp;
	index = i;

	if (item != NULL)
		temp = item;

	lbl = new QLabel("qP:",this);
	topLayout->addWidget(lbl,0,0);
	lbl = new QLabel("MW:",this);
	topLayout->addWidget(lbl,1,0);
	lbl = new QLabel("o椽q:",this);
	topLayout->addWidget(lbl,2,0);

	Title = new QLabel(this);
	Title->setAlignment(AlignCenter);
	topLayout->addMultiCellWidget(Title,0,2,2,2,AlignCenter);
	btn = new QPushButton("Browse",this);
	QToolTip::add( btn, "choose picture");
	connect(btn,SIGNAL(clicked()),this,SLOT(PicBrowse()));
	topLayout->addWidget(btn,1,3);

	QFrame *separator = new QFrame( this, "separatorLine" );
	separator->setFrameStyle( QFrame::HLine | QFrame::Sunken );
	separator->setFixedHeight( separator->sizeHint().height() );
	topLayout->addMultiCellWidget( separator, 3, 3, 0, 3 );

	btn = new QPushButton("Insert",this);
	QToolTip::add( btn, "insert one");
	connect(btn,SIGNAL(clicked()),this,SLOT(InsertSong()));
	topLayout->addWidget(btn,4,0);
	btn = new QPushButton("Remove",this);
	QToolTip::add( btn, "delete one");
	connect(btn,SIGNAL(clicked()),this,SLOT(RemoveSong()));
	topLayout->addWidget(btn,4,1);

	lbl = new QLabel("qW:",this);
	topLayout->addWidget(lbl,4,2);

	edtSinger = new QLineEdit(this);
	topLayout->addWidget(edtSinger,0,1);
	edtAlbum = new QLineEdit(this);
	topLayout->addWidget(edtAlbum,1,1);
	edtPublish = new QLineEdit(this);
	topLayout->addWidget(edtPublish,2,1);

	edtSong = new QLineEdit(this);
	connect(edtSong,SIGNAL(textChanged(const char *)),this,SLOT(SetSongName()));
	topLayout->addWidget(edtSong,4,3);

	FileBox = new QListBox(this);
	topLayout->addMultiCellWidget(FileBox,5,14,0,1);
	connect(FileBox,SIGNAL(highlighted(int)),this,SLOT(FileChange(int)));
	SongBox = new QListBox(this);
	topLayout->addMultiCellWidget(SongBox,5,14,2,3);
	connect(SongBox,SIGNAL(highlighted(int)),this,SLOT(SongChange(int)));

	btn = new QPushButton("Browse",this);
	connect(btn,SIGNAL(clicked()),this,SLOT(FileBrowse()));
	topLayout->addWidget(btn,15,1);
	btn = new QPushButton("Cancel",this);
	connect(btn,SIGNAL(clicked()),this,SLOT(reject()));
	topLayout->addWidget(btn,15,3);
	btn = new QPushButton("OK",this);
	connect(btn,SIGNAL(clicked()),this,SLOT(accept()));
	topLayout->addWidget(btn,15,2);
	

	if (item == NULL){
		Title->setPixmap(QPixmap((const char **)unknown_album_xpm));
		Picture = new QString("unknown.xpm");
		edtSinger->setText("");
	}
	else {
		QPixmap pixmap;
		if (!pixmap.load(temp.GetPicture()))
			pixmap = QPixmap((const char **)unknown_album_xpm);
		Title->setPixmap(pixmap);
		Picture = new QString(temp.GetPicture());
		edtSinger->setText(temp.GetSinger());
		edtAlbum->setText(temp.GetSingerAlbum());
		edtPublish->setText(temp.GetPublish());
		SongBox->insertStrList(temp.GetSongList());
		FileBox->insertStrList(temp.GetFileList());
	}

	topLayout->activate();
	resize(425,390);
}

void AlbumInfoView::FileChange(int i)
{
	SongBox->setCurrentItem(i);
}

void AlbumInfoView::SongChange(int i)
{
	FileBox->setCurrentItem(i);
	edtSong->setText(SongBox->text(i));
}

void AlbumInfoView::SetSongName()
{
	SongBox->changeItem(edtSong->text(),SongBox->currentItem());
}

void AlbumInfoView::PicBrowse()
{
	QString filename = MyOpenFile::getOpenFileName(this);
	if (!filename.isNull()){
		QPixmap p;
		if (!p.load(filename))
			QMessageBox::warning(this,"error","can't open picture file\n");
		else{
			Picture = new QString(filename);
			Title->setPixmap(p);
		}
	}
}

void AlbumInfoView::FileBrowse()
{
	FileBrowseView *fb = new FileBrowseView(this);
	if (fb->exec() == QDialog::Accepted){
		FileBox->clear(); SongBox->clear();
		int i,c;
		QStrList getlist = fb->GetSongList();
		FileBox->insertStrList(&getlist);
		for (i=0;i<FileBox->count();i++){
			QString str = QString(FileBox->text(i));
			if ((c=str.findRev('/')) != -1)
				SongBox->insertItem(str.right(str.length()-c-1));
			else
				SongBox->insertItem(str);
		}
	}
}

AlbumItem AlbumInfoView::GetAlbumInfo()
{
	int i;
        AlbumItem temp;

	temp.Singer = edtSinger->text();
	temp.SingerAlbum = edtAlbum->text();
	temp.Publish = edtPublish->text();
	temp.Picture = *Picture;
	for (i=0;i<FileBox->count();i++)
		temp.AddSong((char *)(SongBox->text(i)),(char *)FileBox->text(i));
	return temp;
}

void AlbumInfoView::InsertSong()
{
	int i = FileBox->currentItem();
	int c;
	QString filename = MyOpenFile::getOpenFileName(this,"*.mp3");
	QString songname;
	if ( i > -1){
        	if (!filename.isNull()){
			FileBox->insertItem(filename,i);
			if ((c=filename.findRev('/')) != -1)
				SongBox->insertItem(filename.right(filename.length()-c-1),i);
			else
				SongBox->insertItem(filename,i);
		}
	}
	else {
		if (!filename.isNull()){
			FileBox->insertItem(filename);
			if ((c=filename.findRev('/')) != -1)
				SongBox->insertItem(filename.right(filename.length()-c-1));
			else
				SongBox->insertItem(filename);
		}
	}
}

void AlbumInfoView::RemoveSong()
{
	int i = FileBox->currentItem();
	if ( i > -1){
		FileBox->removeItem(i);
		SongBox->removeItem(i);
	}
}
