#include <qpainter.h>
#include <stdlib.h>
#include <stdio.h>
#include "MultiListBox.h"

// MultiListBox Implement
// written by mills

MultiListBox::MultiListBox(QWidget *parent=0,char *name=0,int numCols=4)
		:QTableView(parent,name)
{
	ItemList.setAutoDelete(TRUE);
	SetAutoArrange(TRUE);
	curRow = curCol = -1;
	oldRow = oldCol = -2;
	count = 0;
	fixedColWidth = 100;
	setNumCols(numCols);
	setNumRows(1);
	setBackgroundMode(PaletteBase);
	setTableFlags(Tbl_autoVScrollBar | Tbl_autoHScrollBar | Tbl_clipCellPainting | Tbl_smoothScrolling);
	setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
	resize(600,400);
}

void MultiListBox::InsertItem(int row,int col,MultiListBoxItem *item)
{
	InsertItem(indexOf(row,col),item);
}

void MultiListBox::InsertItem(int i,MultiListBoxItem *item)
{
	// deep copy
	if (i < count && i > -1){
		MultiListBoxItem *tmp = new MultiListBoxItem(item);
		ItemList.insert(i,tmp);
		count++;
		if (HintRow() > numRows())
			setNumRows(numRows()+1);
		int j = i;
		for (j;j<count;j++)
			updateCell(j/numCols(),j%numCols());
	}
	else
		printf("%d is out of range %d\n",i,count);
}

void MultiListBox::DeleteItem(int row,int col)
{
	DeleteItem(indexOf(row,col));
}

void MultiListBox::DeleteItem(int i)
{
	if (i<count && i > -1){
		ItemList.remove(i);
		count--;
		if (HintRow() < numRows())
			setNumRows(numRows()-1);
		repaint();
	}
	else
		printf("%d is out of range %d\n",i,count);
}

void MultiListBox::Clear()
{
	setAutoUpdate(FALSE);
	int i;
	int tmp = count;
	for(i=0;i<tmp;i++)
		DeleteItem(0);
	setAutoUpdate(TRUE);
	repaint();
}

void MultiListBox::AppendItem(MultiListBoxItem *item)
{
	MultiListBoxItem *tmp = new MultiListBoxItem(item);
	ItemList.append(tmp);
	count++;
	if (HintRow() > numRows())
		setNumRows(numRows()+1);
	updateCell((count-1)/numCols(),(count-1)%numCols());
}

void MultiListBox::ChangeItem(int row,int col,MultiListBoxItem *item)
{
	ChangeItem(indexOf(row,col),item);
}

void MultiListBox::ChangeItem(int i,MultiListBoxItem *item)
{
	if (i<count && i > -1){
		MultiListBoxItem *tmp = new MultiListBoxItem(item);
		ItemList.remove(i);
		ItemList.insert(i,tmp);
		updateCell(i/numCols(),i%numCols());
	}
	else
		printf("%d is out of range %d\n",i,count);
}

int MultiListBox::HintRow()
{
	int i = count%numCols();
	if (i > 0)
		return count/numCols() + 1;
	else
		return count/numCols();
}

void MultiListBox::paintCell(QPainter* p,int row,int col)
{
	int index = indexOf(row,col);
	if (index < count){
		if (row == curRow && col == curCol){
			(ItemList.at(index))->Paint(p,cellWidth(col),cellHeight(row),1);
		}
		else
			(ItemList.at(index))->Paint(p,cellWidth(col),cellHeight(row),0);
	}
}

int MultiListBox::indexOf(int row,int col)
{
	int i = (row * numCols()) + col;
	if (i<0)
		i=-1;
	return i;
}

int MultiListBox::cellWidth(int col)
{
/* 	//Variable Column Width
	int i;
	int width=0;
	for (i=col;i<count;i+=numCols()){
		int w = ItemList.at(i)->Width(this);
		if (w>width)
			width = w;
	}
	return width;
*/
	return fixedColWidth;
}

int MultiListBox::cellHeight(int row)
{
	int i;
	int height=0;
	for (i=row*numCols();i<count && i < (row+1)*numCols();i++){
		int h = ItemList.at(i)->Height(this,fixedColWidth);	// fixed
		if (h>height)
			height = h;
	}
	return height;
}

void MultiListBox::mousePressEvent(QMouseEvent* e)
{
	int tmp_row,tmp_col;
	oldRow = curRow;
	oldCol = curCol;
	QPoint clickedPos = e->pos();
	curRow = findRow( clickedPos.y() );
	curCol = findCol( clickedPos.x() );

	if ( (curRow != oldRow) || (curCol != oldCol) ) {
		updateCell( oldRow, oldCol );
		updateCell( curRow, curCol );
		if (indexOf(curRow,curCol) < count && indexOf(curRow,curCol) > -1)
//			printf("highlight on %d (%d,%d)\n",indexOf(curRow,curCol),curRow,curCol);
			emit highlighted(indexOf(curRow,curCol));
	}
	if (curRow < 0 || curCol <0){
		curRow = -1;
		curCol = -1;
	}
}

void MultiListBox::mouseDoubleClickEvent(QMouseEvent* e)
{
	if (indexOf(curRow,curCol) < count && indexOf(curRow,curCol) > -1)
//		printf("selected on %d (%d,%d)\n",indexOf(curRow,curCol),curRow,curCol);
		emit selected(indexOf(curRow,curCol));
}

void MultiListBox::resizeEvent(QResizeEvent *e)
{
	if (isAutoArrange == TRUE){
		int i;
		int newWidth = viewWidth();
		int tmpWidth =0;

		for (i=0;;i++){
			tmpWidth += cellWidth(i);
			if (tmpWidth > newWidth)
				break;
		}
		if (i<1)
			i=1;
		setNumCols(i);
		setNumRows(HintRow());
		updateTableSize();
	}
}


// MultiListBoxItem Implement
// written by mills

int MultiListBoxItem::Width(MultiListBox *box)
{
/*
	int base_w = Pic.width();
	int result = QMAX(base_w,box->fontMetrics().width(Text));
	return result + 30 ;
*/
	printf("function is cancelled!\n");
}

int MultiListBoxItem::Height(MultiListBox *box,int max_width)
{
	int base_h = Pic.height();
	int font_h = box->fontMetrics().height();
	if (box->fontMetrics().width(Text) > max_width)
		font_h *= 2;
	int result = base_h + font_h;
	return result + 20 ;
}

void MultiListBoxItem::Paint(QPainter *p, int max_w, int max_h,int flag=0)
{
	int base_h = Pic.height();
	int pos_x,pos_y;
	int tmp;
	int font_h = p->fontMetrics().height();
	int font_w = p->fontMetrics().width(Text);
	pos_x = (max_w - font_w)/2;
	tmp = base_h + font_h;
	pos_y = (max_h - tmp)/2 + tmp;
	if (flag){
		p->setBackgroundColor(QColor(black));
		p->setPen(QColor(white));
		p->setBackgroundMode(OpaqueMode);
		MyDrawText(p,pos_x,pos_y,max_w);
		p->setBackgroundColor(QColor(white));		
		p->setPen(QColor(black));
		p->setBackgroundMode(TransparentMode);
	}
	else
		MyDrawText(p,pos_x,pos_y,max_w);

	pos_x = (max_w-Pic.width())/2;
	pos_y = (max_h - tmp)/2;
	p->drawPixmap(pos_x,pos_y,Pic);

//	p->drawRect(0,0,max_w,max_h);
}

void MultiListBoxItem::MyDrawText(QPainter *p,int x,int y,int max_w)
{
	if (x<0){		// text is too long
        	int font_h = p->fontMetrics().height();
        	int font_w = p->fontMetrics().width(Text);
		int char_w = p->fontMetrics().width("a");	// test width of a character
		int i;
		for (i=0;i<Text.length();i++)
			if (p->fontMetrics().width(Text.left(i)) > max_w)
				break;
		int num = i - 1 ;

		char *buf = (char *)malloc(Text.length());
		strcpy(buf,Text.data());
		for (i=0;i<num;i++){		// check chinese
			unsigned int c = (unsigned int) buf[i];
			if (c > 127)
				i++;
		}
		if (i>num)			// is chinese break
			num--;
		free(buf);
		QString str1 = Text.left(num);
		QString str2 = Text.right(Text.length()-num);
		p->drawText(0,y,str1.data());
		int str2_w = p->fontMetrics().width(str2);
		if (str2_w > max_w)
			p->drawText(0,y+font_h,str2.data());
		else
			p->drawText((max_w-str2_w)/2,y+font_h,str2.data());
	}
	else
		p->drawText(x,y,Text.data());
}
