#include <qpainter.h> 
#include <qpixmap.h> 
#include <stdio.h>
#include "clock.h" 
#include "clock.res"

#define NUM_LEFT 5
#define NUM_TOP  15
#define CLOCK_LEFT 60
#define CLOCK_TOP 32
#define CELL_W 15
#define CELL_H 12

DigitalClock::DigitalClock( QWidget *parent, const char *name,int a, int b)
    : QFrame( parent, name )
{
	MaxSong = 12;
	CurrSong = 0;
	isStop = 1;
	isPause = 0;
	isBlank = 1;
	flag=0;
	isRepeat = a;
	isAutoPlay = b;
	setFrameStyle( QFrame::Panel | QFrame::Sunken );
	oldtime = newtime = "-:--:--";
	normalTimer = startTimer( 1000 );
}

void DigitalClock::timerEvent( QTimerEvent *e )
{
	if (isStop == 0 && isPause != 1){
	        mytime = mytime.addSecs(1);
		ShowTime();
	}
	if (isStop == 0 && isPause == 1)
		ShowTime();
}

void DigitalClock::ShowTime()
{
	QString s;

	if (isPause){
		if (isBlank)
			s = "       ";
		else
			s = mytime.toString();			
		isBlank = !isBlank;
	}
	else
		s = mytime.toString();
	display(s);
}

void DigitalClock::Start(int i)
{
	if (isStop != 1)
		Stop();
	isStop = 0;
	mytime.setHMS(0,0,0);

	QPainter p;
	int pos_x,pos_y;

	if (i > 12*(flag+1)-1 || i < 12*flag)
		SetMax(MaxSong,i/12);

	p.begin(this);
	p.setRasterOp(XorROP);
	p.setBrush(QBrush(green));
	if (CurrSong != -1){
		p.drawRoundRect(NUM_LEFT-2+CELL_W*(CurrSong%3),NUM_TOP-11+CELL_H*(CurrSong/3),CELL_W,CELL_H+1,0,0);
	}
	CurrSong=i%12;
	p.drawRoundRect(NUM_LEFT-2+CELL_W*(CurrSong%3),NUM_TOP-11+CELL_H*(CurrSong/3),CELL_W,CELL_H+1,0,0);
	p.end();
}

void DigitalClock::SetMax(int i,int f)
{
	MaxSong = i;
	flag = f;
	char buf[3];
	QPainter p;
	int pos_x,pos_y;
	CurrSong=-1;
	int num = MaxSong - 12*flag;

	p.begin(this);
	p.fillRect(NUM_LEFT-2,NUM_TOP-CELL_H,CELL_W*3,CELL_H*4+1,QBrush(QColor(green)));
	p.setFont(QFont("Helvetica",10));
	for (i=0;i<12 && i<num;i++){
		pos_x=NUM_LEFT+CELL_W*(i%3);
		pos_y=NUM_TOP+CELL_H*(i/3);
		sprintf(buf,"%d",i+1+12*flag);
		if (strlen(buf) == 1)
			pos_x += 3;
		p.drawText(pos_x,pos_y,buf,2);
	}
	p.end();
}

void DigitalClock::Stop()
{
	isPause = 0;
	isStop=1;
	QString s = "-:--:--";
	display(s);
}

void DigitalClock::display(QString str)
{
	QPainter p1,p2;
	QPixmap TmpClock;
	newtime = str.right(7);

	TmpClock.resize(width()-CLOCK_LEFT-1,30);
	p1.begin(&TmpClock);
	p1.setFont(QFont("Helvetica",32));
	p1.fillRect(0,0,TmpClock.width(),TmpClock.height(),QBrush(QColor(green)));
	p1.setPen(QPen(red,2));
	p1.drawText(0,30,newtime.data());
	p1.end();

	p2.begin(this);
	p2.drawPixmap(CLOCK_LEFT,CLOCK_TOP-TmpClock.height(),TmpClock);
	p2.end();
	oldtime=newtime;
}

void DigitalClock::drawContents(QPainter *p)
{
	int i,pos_x,pos_y;
	char buf[3];

	// draw clock
        p->setFont(QFont("Helvetica",32));
        p->setPen(QPen(red,2));
        p->drawText(CLOCK_LEFT,CLOCK_TOP,newtime.data());

	// draw number
	p->setPen(QPen(black,1));
	p->setFont(QFont("Helvetica",10));
	int num = MaxSong - 12*flag;
	for (i=0;i<12 && i<num;i++){
		pos_x=NUM_LEFT+CELL_W*(i%3);
		pos_y=NUM_TOP+CELL_H*(i/3);
		sprintf(buf,"%d",i+1+12*flag);
		if (strlen(buf) == 1)
			pos_x += 3;
		p->drawText(pos_x,pos_y,buf);
	}

	// draw current
        p->setRasterOp(XorROP);
        p->setBrush(QBrush(green));
	p->drawRoundRect(NUM_LEFT-2+CELL_W*(CurrSong%3),NUM_TOP-11+CELL_H*(CurrSong/3),CELL_W,CELL_H+1,0,0);

	// draw icon
	p->setRasterOp(CopyROP);
	QPixmap pix;
	if (isAutoPlay)
		pix = QPixmap((const char **)p_autoplay_on);
	else
		pix = QPixmap((const char **)p_autoplay_off);
	p->drawPixmap(130,40,pix);

	if (isRepeat)
		pix = QPixmap((const char **)p_repeat_on);
	else
		pix = QPixmap((const char **)p_repeat_off);
	p->drawPixmap(100,40,pix);
}

void DigitalClock::SetRepeat(int i)
{
	QPainter p;
	QPixmap pix;

	p.begin(this);
	if (i)
		pix = QPixmap((const char **)p_repeat_on);
	else
		pix = QPixmap((const char **)p_repeat_off);
	p.drawPixmap(100,40,pix);
	p.end();
	isRepeat = i ;
}

void DigitalClock::SetAutoPlay(int i)
{
	QPainter p;
	QPixmap pix;

	p.begin(this);
	if (i)
		pix = QPixmap((const char **)p_autoplay_on);
	else
		pix = QPixmap((const char **)p_autoplay_off);
	p.drawPixmap(130,40,pix);
	p.end();
	isAutoPlay = i;
}

void DigitalClock::Pause()
{
	if (isPause == 1)
		isPause = 0;
	else
		isPause = 1;
}
