#include <qapp.h>
#include <qcolor.h>
#include <qpushbt.h>
#include <qlayout.h>
#include <qmlined.h>
#include <qmenubar.h>
#include <qlistbox.h>
#include <qstring.h>
#include <qmsgbox.h>
#include <qslider.h>
#include <qtooltip.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>
#include "player.h"
#include "filebrowse.h"
#include "clock.h"
#include "player.res"

#define MP3_PLAYER "mpg123"

int PlayerWidget::mp3_pid = 0;
int PlayerWidget::status = 0;
int PlayerWidget::isContinue=0;
int PlayerWidget::isStop=1;
int PlayerWidget::isPause = 0;
int PlayerWidget::CurrPlaySong = -2;

void PlayerWidget::ChildEnd(int s)
{
	if (!isPause){
		wait(&status);
		mp3_pid = 0;
		isStop = 1;
	}
}

PlayerWidget::PlayerWidget( QWidget *parent, const char *name )
		: QWidget( parent, name )
{
	isRepeat=0;
	isAutoPlay = 1;
	MyMixer = new Mixer();

	signal(SIGCHLD,ChildEnd);
	startTimer( 1000 );

	setCaption("MusicBox-1.01");

	list = new AlbumListBox(this);
	connect(list,SIGNAL(selected(AlbumItem)),this,SLOT(AlbumChange(AlbumItem)));
	list->LoadFromFile();
	list->setGeometry(396,34,150,211);

	QMenuBar *menubar = new QMenuBar( this );
	menubar->setSeparator( QMenuBar::InWindowsStyle );
	QPopupMenu *popup1 = new QPopupMenu;
	popup1->insertItem( "&New Album", list, SLOT(NewAlbum()) );
	popup1->insertItem( "&Open Files", this, SLOT(OpenFiles()));
	popup1->insertItem( "&Save", list, SLOT(SaveToFile()) );
	popup1->insertItem( "&Quit", qApp, SLOT(quit()) );
	menubar->insertItem( "&File", popup1 );

	QPopupMenu *popup2 = new QPopupMenu;
	popup2->insertItem( "&Small Panel", this, SLOT(SmallView()) );
	popup2->insertItem( "&Normal Panel", this, SLOT(NormalView()) );
	popup2->insertItem( "&Completed Panel", this, SLOT(AllView()) );
	menubar->insertItem( "&View", popup2 );

	options = new QPopupMenu;
	AutoPlayID = options->insertItem( "&Auto Play", this, SLOT(AutoPlay()) );
	options->setItemChecked( AutoPlayID, isAutoPlay );
	RepeatID = options->insertItem( "Re&peat", this, SLOT(RepeatPlay()) );
	options->setCheckable(TRUE);
	menubar->insertItem( "&Options",options );
	menubar->insertSeparator();
	QPopupMenu *help = new QPopupMenu;
	help->insertItem("&How to use?",this,SLOT(Usage()));
	help->insertItem("&About",this,SLOT(About()));
	menubar->insertItem("&Help",help);

	QLabel *title = new QLabel(this);
	title->setPixmap(QPixmap(title_xpm));
	title->setGeometry(5,34,65,92);

	QPushButton* but;

	QPixmap p_first((const char **)first_xpm);
	QPixmap p_pre((const char **)previous_xpm);
	QPixmap p_play((const char **)play_xpm);
	QPixmap p_stop((const char **)stop_xpm);
	QPixmap p_next((const char **)next_xpm);
	QPixmap p_last((const char **)last_xpm);

	but = new QPushButton("First",this);
	but->setPixmap(p_first);
	QToolTip::add( but, "First");
	connect(but,SIGNAL(clicked()),this,SLOT(GoFirst()));
	but->resize(60,30); but->move(248,34);

	but =  new QPushButton("Previous",this);
	but->setPixmap(p_pre);
	QToolTip::add( but, "Previous");
	connect(but,SIGNAL(clicked()),this,SLOT(GoPrevious()));
	but->resize(60,30); but->move(248,96);

	but = new QPushButton("Play",this);
	but->setPixmap(p_play);
	QToolTip::add( but, "Play/Pause");
	connect(but,SIGNAL(clicked()),this,SLOT(GoPlay()));
	but->resize(60,30); but->move(248,65);

	but = new QPushButton("Stop",this);
	but->setPixmap(p_stop);
	QToolTip::add( but, "Stop");
	connect(but,SIGNAL(clicked()),this,SLOT(GoStop()));
	but->resize(60,30); but->move(308,65);

	but = new QPushButton("Next",this);
	but->setPixmap(p_next);
	QToolTip::add( but, "Next");
	connect(but,SIGNAL(clicked()),this,SLOT(GoNext()));
	but->resize(60,30); but->move(308,96);

	but = new QPushButton("Last",this);
	but->setPixmap(p_last);
	QToolTip::add( but, "Last");
	connect(but,SIGNAL(clicked()),this,SLOT(GoLast()));
	but->resize(60,30); but->move(308,34);

	dclock = new DigitalClock( this,"status",isRepeat,isAutoPlay);
	dclock->setBackgroundColor(green);
	dclock->setGeometry(67,34,179,61);

	song = new QLabel( "", this );
	song->setBackgroundColor( green );
	song->setAlignment(AlignCenter);
	song->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	song->setGeometry(67,96,179,30);

	int orig_sound = MyMixer->Get();
	QSlider *volume = new QSlider(0,100,10,(orig_sound == -1)?50:100-orig_sound,QSlider::Vertical,this,"volume");
	volume->setTickmarks( QSlider::Right );
	volume->setFixedWidth(volume->sizeHint().width());
	QToolTip::add( volume, "Volume");
	connect(volume,SIGNAL(valueChanged(int)),this,SLOT(VolumeChange(int)));
	volume->setGeometry(370,34,24,92);

	QLabel *label;
	label = new QLabel( this );
	label->setText("qP:");
	label->setGeometry(5,133,125,30);

	edtName = new QLabel( this );
	edtName->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	edtName->setGeometry(83,136,100,25);

	label = new QLabel( this );
	label->setText("MW:");
	label->setGeometry(5,173,125,30);

	edtAlbum = new QLabel( this );
	edtAlbum->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	edtAlbum->setGeometry(83,176,100,25);

	label = new QLabel( this );
	label->setText("o椽q:");
	label->setGeometry(5,213,125,30);

	edtPub = new QLabel( this );
	edtPub->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	edtPub->setGeometry(83,216,100,25);

	SongBox = new QListBox( this );
	LocBox = new QListBox();
	connect(SongBox,SIGNAL(highlighted(int)),this,SLOT(GoPlay()));
	SongBox->setGeometry(192,133,195,112);

	AllView();
}
void PlayerWidget::resizeEvent(QResizeEvent *e)
{
	list->setGeometry(396,34,width()-396-5,height()-34-5);
	SongBox->setGeometry(192,133,195,height()-133-5);
}

void PlayerWidget::RepeatPlay()
{
	isRepeat = !isRepeat;
	options->setItemChecked( RepeatID, isRepeat );
	dclock->SetRepeat(isRepeat);
}

void PlayerWidget::AutoPlay()
{
	isAutoPlay = !isAutoPlay;
	options->setItemChecked( AutoPlayID, isAutoPlay );
	dclock->SetAutoPlay(isAutoPlay);
}

void PlayerWidget::GoFirst()
{
	if (SongBox->count()>0)
		SongBox->setCurrentItem(0);
}

void PlayerWidget::GoPrevious()
{
	if (SongBox->currentItem() > 0)
		SongBox->setCurrentItem(SongBox->currentItem()-1);
}

void PlayerWidget::GoPlay()
{
	if (SongBox->count()>0 && SongBox->currentItem() < 0)
		SongBox->setCurrentItem(0);
	else if (SongBox->count()>0 && mp3_pid != 0 && CurrPlaySong == SongBox->currentItem()){
		if (isPause){
			isPause=0;
			kill(mp3_pid,SIGCONT);
			dclock->Pause();
		}
		else{
			isPause=1;
			kill(mp3_pid,SIGSTOP);
			dclock->Pause();
		}
	}
	else if (SongBox->count()>0) {
		GoStop();
		if (access(LocBox->text(SongBox->currentItem()),R_OK)== -1){
			QString s = "can't find ";
			s.append(LocBox->text(SongBox->currentItem()));
			QMessageBox::warning(this,"error",s.data());
			return;
		}
		isContinue = 1;
		isStop = 0 ;
		CurrPlaySong = SongBox->currentItem();
		song->setText(SongBox->text(SongBox->currentItem()));
		dclock->Start(SongBox->currentItem());
		if ((mp3_pid = fork()) == 0 ){
			char **argv;
			int i;
			argv[0]=MP3_PLAYER;
			argv[1]="-q";
			argv[2]=(char *)LocBox->text(SongBox->currentItem());
			argv[3]=NULL;
			execvp(MP3_PLAYER,argv);
			QMessageBox::warning(this,"error","can't exec mpg123\n");
			exit(1);
		}
	}
}

void PlayerWidget::GoStop()
{
	isPause=0;
	isStop = 1;
	isContinue = 0;
	if (mp3_pid != 0){
		if (!kill(mp3_pid,SIGKILL))
			while(mp3_pid != 0);
		else
			QMessageBox::warning(this,"error","can't kill mpg123 child process\n");
	}
	dclock->Stop();
	song->setText("");
}

void PlayerWidget::GoNext()
{
	if (SongBox->currentItem() < SongBox->count()-1){
		SongBox->setCurrentItem(SongBox->currentItem()+1);
	}
}

void PlayerWidget::GoLast()
{
	if (SongBox->count()>0)
		SongBox->setCurrentItem(SongBox->count()-1);
}

void PlayerWidget::AlbumChange(AlbumItem ai)
{
	int i;
        GoStop();
        SongBox->clear();
	LocBox->clear();

	AlbumItem *test = new AlbumItem;
	*test = ai;
	SongBox->insertStrList(test->GetSongList());
	LocBox->insertStrList(test->GetFileList());

	edtName->setText(test->GetSinger());
	edtAlbum->setText(test->GetSingerAlbum());
	edtPub->setText(test->GetPublish());

	dclock->SetMax(SongBox->count(),0);
	if (isAutoPlay && SongBox->count() > 0)
		SongBox->setCurrentItem(0);
}

void PlayerWidget::VolumeChange(int level)
{
	MyMixer->Set(100-level);
}

void PlayerWidget::About()
{
	QMessageBox::about( this, "About this...",
			"MusicBox 1.01 Patch Level 2\n\n"
                        "This is a freeware MP3 player .\n"
                        "You can distrubute it freely.\n"
                        "Author: mills@iim.nctu.edu.tw\n"
			"	(NCTU-IIM)" );
}

void PlayerWidget::SmallView()
{
	resize(395,132);
}

void PlayerWidget::AllView()
{
	resize(730,400);
}

void PlayerWidget::NormalView()
{
	resize(395,250);
}

void PlayerWidget::OpenFiles()
{
	FileBrowseView *of = new FileBrowseView(this);
	if (of->exec() == QDialog::Accepted){
		AlbumItem ai;
		int c;
		QStrList song = of->GetSongList();
		if (song.isEmpty())
			return;
		QString str;
		for (str=song.first();!str.isNull();str=song.next()){
			if ((c=str.findRev('/')) != -1)
				ai.AddSong(str.right(str.length()-c-1),str);
			else
				ai.AddSong(str,str);
		}
		delete of;
		AlbumChange(ai);
	}
	else
		delete of;
}

void PlayerWidget::Usage()
{
	char help_txt[] = "
1. Menu:\n
1.1 File\n
    New Album
	Create an new Album for playing later.
    Open Files
	Select some mp3 files you like, and play them
        in sequence.
    Save
	Save all album in the right frame to file.
    Quit
	Quit MusicBox.\n
1.2 View\n
    Small Panel
	view controls only.
    Normal Panel
	view controls and song list.
    Completed Panel
	view all frames.\n
1.3 Options\n
    Auto Play
	Auto Play songs after choosing an album or files.
    Repeat
	Rewind, and play them again.\n
1.4 Help\n
    Some information about music box\n
2. Albums:\n
    Album is a set of songs, you can add name of song to
    relative file, and singer, album name, and publisher.
    Make it more readable.\n
    You can choose an icon to represent an album.GIF,
    XPM,BMP is supported, The size 48x48 is prefered.\n
    If you want to edit album information, click mouse 
    right-button and click edit.\n
3. Usage:\n
    You can choose files to play or play album.\n
*Important*\n
    \"mpg123\" package must be installed first before using
    MusicBox.
";

	help = new QWidget (0,"help");
	QVBoxLayout *toplayout = new QVBoxLayout(help,5);
	QMultiLineEdit *edit = new QMultiLineEdit(help,"use");
	toplayout->addWidget(edit);
	help->setCaption("How to use...");
	edit->setReadOnly(TRUE);
	edit->setText(help_txt);
	QPushButton *ok = new QPushButton("OK",help);
	ok->setFixedSize(75,30);
	toplayout->addWidget(ok);
	connect(ok,SIGNAL(clicked()),this,SLOT(closehelp()));
	help->resize(350,390);

	toplayout->activate();
	help->show();
}

void PlayerWidget::closehelp()
{
	delete help;
}


PlayerWidget::~PlayerWidget()
{
	GoStop();
	if ( list->IsChange() == 1)
		switch( QMessageBox::information( this, "Save?",
                                      "Your Album file is changed\n"
                                      "Save Now?",
                                      "&Save", "&Don't Save", "&Cancel",
                                      0,2)){
    		case 0:
			if(!list->SaveToFile())
				QMessageBox::warning(this,"error","can't save!\n");
        		break;
    		case 1:
		case 2:
        		break;
		}
}

void PlayerWidget::timerEvent( QTimerEvent *e )
{
//	printf("isContinue:%d isStop:%d isRepeat:%d\n",isContinue,isStop,isRepeat);
	if (isStop && !isContinue && mp3_pid == 0)
		;				// Stop State, In Idle
	else if(!isStop && isContinue && mp3_pid != 0)
		;				// Playing
	else if (SongBox->currentItem() == (SongBox->count()-1) && !isRepeat && mp3_pid == 0)
		GoStop();			// End
	else if (SongBox->currentItem() == (SongBox->count()-1) && isRepeat && mp3_pid == 0){
		if (SongBox->count() == 1){	// End and re-play
			GoStop();
			GoPlay();
		}
		else
			SongBox->setCurrentItem(0);
	}
	else if (isStop && isContinue && mp3_pid == 0 )
		GoNext();			// Continuously play
	else 
		printf("Unknow state: isContinue:%d isStop:%d isRepeat:%d MP3_PID:%d\n",isContinue,isStop,isRepeat,mp3_pid);
}

int main( int argc, char **argv )
{
	QApplication a( argc, argv );
	QApplication::setFont( QFont("Helvetica") );

	QWidget *f = new PlayerWidget;
	f->show();

	a.setMainWidget(f);
	return a.exec();
}
